@@ -253,13 +253,13 @@ fn mutate_error_both_lock_f(task: &mut FTask2) {
     cond.notify_all();
 }
 
-fn fetch_block_s(db: &DBCopy, task: STask) {
+fn fetch_block_s(db: &DBCopy, task: STask) -> bool {
     // check error state at beginning, preventing new works generated
     // which might result in deadlock
     if task.error_state.load(Ordering::SeqCst) {
         // should never increment lock condition here!! Otherwise later tasks might skip
         // over unfinished tasks
-        return;
+        return false;
     }
     let task_number = task.task_number;
 
@@ -280,32 +280,33 @@ fn fetch_block_s(db: &DBCopy, task: STask) {
                         *output_number += 1;
                         cond.notify_all();
                         // before return, always increase lock condition
-                        return;
+                        return false;
                     }
                     task.sender.send(SBlock::parse(blk)).unwrap();
                     *output_number += 1;
                     cond.notify_all();
                 }
+                true
             }
             Err(_) => {
                 mutate_error_state_s(task);
-                return;
+                return false;
             }
         }
     } else {
         // set error_state to true
         mutate_error_state_s(task);
-        return;
+        return false;
     }
 }
 
-fn fetch_block_f(db: &DBCopy, task: FTask) {
+fn fetch_block_f(db: &DBCopy, task: FTask) -> bool {
     // check error state at beginning, preventing new works generated
     // which might result in deadlock
     if task.error_state.load(Ordering::SeqCst) {
         // should never increment lock condition here!! Otherwise later tasks might skip
         // over unfinished tasks
-        return;
+        return false;
     }
     let task_number = task.task_number;
 
@@ -326,22 +327,23 @@ fn fetch_block_f(db: &DBCopy, task: FTask) {
                         *output_number += 1;
                         cond.notify_all();
                         // before return, always increase lock condition
-                        return;
+                        return false;
                     }
                     task.sender.send(FBlock::parse(blk)).unwrap();
                     *output_number += 1;
                     cond.notify_all();
                 }
+                true
             }
             Err(_) => {
                 mutate_error_state_f(task);
-                return;
+                return false;
             }
         }
     } else {
         // set error_state to true
         mutate_error_state_f(task);
-        return;
+        return false;
     }
 }
 
@@ -349,12 +351,12 @@ fn fetch_fblock_connected(
     mut unspent: &Arc<Mutex<HashMap<Txid, Arc<Mutex<FVecMap>>>>>,
     db: &DBCopy,
     mut task: FTask2,
-) {
+) -> bool {
     // stop new tasks from loading
     if task.error_state.load(Ordering::SeqCst) {
         // should never increment lock condition here!! Otherwise later tasks might skip
         // over unfinished tasks
-        return;
+        return false;
     }
     let my_height = task.height;
 
@@ -414,7 +416,7 @@ fn fetch_fblock_connected(
                         let mut result_height = lock.lock().unwrap();
                         *result_height += 1;
                         cond2.notify_all();
-                        return;
+                        return false;
                     }
                     *outputs_insertion_height += 1;
                     cond.notify_all();
@@ -465,17 +467,13 @@ fn fetch_fblock_connected(
                                 warn!("cannot find previous outpoint, bad data");
                                 // only increment result lock
                                 increment_result_lock_f(&mut task);
-                                return;
+                                return false;
                             }
                         } else {
                             warn!("cannot find previous transactions, bad data");
-                            {
-                                let err = task.error_state.borrow_mut();
-                                err.fetch_or(true, Ordering::SeqCst);
-                            }
                             // only increment result lock
                             increment_result_lock_f(&mut task);
-                            return;
+                            return false;
                         }
                     }
                     output_block.txdata.push(output_tx);
@@ -493,21 +491,24 @@ fn fetch_fblock_connected(
                     if task.error_state.load(Ordering::SeqCst) {
                         *result_height += 1;
                         cond.notify_all();
-                        return;
+                        return false;
                     }
                     task.sender.send(output_block).unwrap();
                     *result_height += 1;
                     cond.notify_all();
                 }
+                true
             }
             Err(_) => {
                 // set error_state to true
                 mutate_error_both_lock_f(&mut task);
+                false
             }
         }
     } else {
         // set error_state to true
         mutate_error_both_lock_f(&mut task);
+        false
     }
 }
 
@@ -515,12 +516,12 @@ fn fetch_sblock_connected(
     mut unspent: &Arc<Mutex<HashMap<Txid, Arc<Mutex<SVecMap>>>>>,
     db: &DBCopy,
     mut task: STask2,
-) {
+) -> bool {
     // stop new tasks from loading
     if task.error_state.load(Ordering::SeqCst) {
         // should never increment lock condition here!! Otherwise later tasks might skip
         // over unfinished tasks
-        return;
+        return false;
     }
     let my_height = task.height;
 
@@ -578,7 +579,7 @@ fn fetch_sblock_connected(
                         let mut result_height = lock.lock().unwrap();
                         *result_height += 1;
                         cond2.notify_all();
-                        return;
+                        return false;
                     }
                     *outputs_insertion_height += 1;
                     cond.notify_all();
@@ -630,17 +631,13 @@ fn fetch_sblock_connected(
                                 }
                                 // only increment result lock
                                 increment_result_lock_s(&mut task);
-                                return;
+                                return false;
                             }
                         } else {
                             warn!("cannot find previous transactions, bad data");
-                            {
-                                let err = task.error_state.borrow_mut();
-                                err.fetch_or(true, Ordering::SeqCst);
-                            }
                             // only increment result lock
                             increment_result_lock_s(&mut task);
-                            return;
+                            return false;
                         }
                     }
                     output_block.txdata.push(output_tx);
@@ -658,21 +655,24 @@ fn fetch_sblock_connected(
                     if task.error_state.load(Ordering::SeqCst) {
                         *result_height += 1;
                         cond.notify_all();
-                        return;
+                        return false;
                     }
                     task.sender.send(output_block).unwrap();
                     *result_height += 1;
                     cond.notify_all();
                 }
+                true
             }
             Err(_) => {
                 // set error_state to true
                 mutate_error_both_lock_s(&mut task);
+                false
             }
         }
     } else {
         // set error_state to true
         mutate_error_both_lock_s(&mut task);
+        false
     }
 }
 pub struct SBlockIteratorArray {
@@ -732,7 +732,11 @@ impl SBlockIteratorArray {
                         match task {
                             // finish
                             None => break,
-                            Some(task) => fetch_block_s(&db_copy, task),
+                            Some(task) => {
+                                if !fetch_block_s(&db_copy, task) {
+                                    break;
+                                }
+                            }
                         }
                     }
                 });
@@ -849,7 +853,11 @@ impl FBlockIteratorArray {
                         match task {
                             // finish
                             None => break,
-                            Some(task) => fetch_block_f(&db_copy, task),
+                            Some(task) => {
+                                if !fetch_block_f(&db_copy, task) {
+                                    break;
+                                }
+                            }
                         }
                     }
                 });
@@ -968,7 +976,11 @@ impl SConnectedBlockIterator {
                         match task {
                             // finish
                             None => break,
-                            Some(task) => fetch_sblock_connected(&unspent_copy, &db_copy, task),
+                            Some(task) => {
+                                if !fetch_sblock_connected(&unspent_copy, &db_copy, task) {
+                                    break;
+                                }
+                            }
                         }
                     }
                 });
@@ -1061,7 +1073,11 @@ impl FConnectedBlockIterator {
                         match task {
                             // finish
                             None => break,
-                            Some(task) => fetch_fblock_connected(&unspent_copy, &db_copy, task),
+                            Some(task) => {
+                                if !fetch_fblock_connected(&unspent_copy, &db_copy, task) {
+                                    break;
+                                }
+                            }
                         }
                     }
                 });
